# Generated by Django 3.2.18 on 2023-12-21 14:29

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("mozilla_django_oidc_db", "0012_alter_openidconnectconfig_sync_groups"),
        ("mozilla_django_oidc_db", "0012_openidconnectconfig_superuser_group_names"),
    ]

    operations = []



# Generated by Django 3.2.23 on 2024-01-09 10:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mozilla_django_oidc_db", "0013_merge_20231221_1529"),
    ]

    operations = [
        migrations.AlterField(
            model_name="openidconnectconfig",
            name="groups_claim",
            field=models.CharField(
                blank=True,
                default="roles",
                help_text="The name of the OIDC claim that holds the values to map to local user groups.",
                max_length=50,
                verbose_name="groups claim",
            ),
        ),
    ]



# Generated by Django 3.2.18 on 2023-12-21 11:59

from django.db import migrations, models

import django_jsonform.models.fields


class Migration(migrations.Migration):
    dependencies = [
        (
            "mozilla_django_oidc_db",
            "0011_alter_openidconnectconfig_userinfo_claims_source",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="openidconnectconfig",
            name="superuser_group_names",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(
                    max_length=50, verbose_name="Superuser group name"
                ),
                blank=True,
                default=list,
                help_text="If any of these group names are present in the claims upon login, the user will be marked as a superuser. If none of these groups are present the user will lose superuser permissions.",
                size=None,
                verbose_name="Superuser group names",
            ),
        ),
    ]



from .typing import ClaimPath


class OIDCProviderOutage(Exception):
    pass


class MissingIdentifierClaim(Exception):
    def __init__(self, claim_bits: ClaimPath, *args, **kwargs):
        self.claim_bits = claim_bits
        super().__init__(*args, **kwargs)



# Generated by Django 4.2.15 on 2024-10-25 14:15

from django.db import migrations
import django_jsonform.models.fields
import mozilla_django_oidc_db.models


class Migration(migrations.Migration):

    dependencies = [
        ("mozilla_django_oidc_db", "0004_remove_openidconnectconfig_oidc_exempt_urls"),
    ]

    operations = [
        migrations.AlterField(
            model_name="openidconnectconfig",
            name="claim_mapping",
            field=django_jsonform.models.fields.JSONField(
                default=mozilla_django_oidc_db.models.get_claim_mapping,
                help_text="Mapping from user-model fields to OIDC claims",
                verbose_name="claim mapping",
            ),
        ),
    ]



from typing import Any, cast

from django.urls import reverse

from mozilla_django_oidc.middleware import SessionRefresh as BaseSessionRefresh
from typing_extensions import override

from .config import (
    BadRequest,
    dynamic_setting,
    get_setting_from_config,
    lookup_config,
)
from .models import OpenIDConnectConfigBase


class SessionRefresh(BaseSessionRefresh):
    """
    Refresh stale sessions based on a config dynamically resolved from the session.
    """

    _config: OpenIDConnectConfigBase
    OIDC_EXEMPT_URLS = dynamic_setting[list[str]](default=[])
    OIDC_OP_AUTHORIZATION_ENDPOINT = dynamic_setting[str]()
    OIDC_RP_CLIENT_ID = dynamic_setting[str]()
    OIDC_STATE_SIZE = dynamic_setting[int](default=32)
    OIDC_AUTHENTICATION_CALLBACK_URL = dynamic_setting[str](
        default="oidc_authentication_callback"
    )
    OIDC_RP_SCOPES = dynamic_setting[str](default="openid email")
    OIDC_USE_NONCE = dynamic_setting[bool](default=True)
    OIDC_NONCE_SIZE = dynamic_setting[int](default=32)

    def __init__(self, get_response):
        # `super().__init__` is not called here, because it calls self.get_setting()
        # The retrieval of these settings is handled via dynamic settings above.
        super(BaseSessionRefresh, self).__init__(get_response=get_response)

    @override
    def get_settings(self, attr: str, *args: Any) -> Any:  # type: ignore
        """
        Look up the request setting from the database config.

        We cache the resolved config instance on the middleware instance for when
        settings are repeatedly looked up. Note however, that we also override the
        __call__ method to delete this cached property, as a middleware instance lives
        for the entire lifecycle of the applicatation, and each request must not look
        at stale cached configuration.
        """
        if (config := getattr(self, "_config", None)) is None:
            raise BadRequest("No config object was set from the request")

        return get_setting_from_config(config, attr, *args)

    def _set_config_from_request(self, request):
        config_class = lookup_config(request)

        # django-solo and type checking is challenging, but a new release is on the
        # way and should fix that :fingers_crossed:
        config = cast(OpenIDConnectConfigBase, config_class.get_solo())
        self._config = config

    def process_request(self, request):
        try:
            self._set_config_from_request(request)
        except BadRequest:
            return None

        # do nothing if the configuration is not enabled
        if not self.get_settings("ENABLED"):
            return None

        return super().process_request(request)

    @property
    def exempt_urls(self):
        # In many cases, the OIDC_AUTHENTICATION_CALLBACK_URL will be the generic
        # callback handler and already be part of super().exempt_urls. However, this is
        # not a given, and consumers might have implemented different callback handlers,
        # in which case they may have overridden the callback URL on the config class.
        #
        # If this is the case, it should always be part of the exempt URLs.
        callback_url = self.OIDC_AUTHENTICATION_CALLBACK_URL
        return {
            callback_url if callback_url.startswith("/") else reverse(callback_url)
        } | super().exempt_urls



from django_setup_configuration.configuration import BaseConfigurationStep
from django_setup_configuration.exceptions import ConfigurationRunFailed

from mozilla_django_oidc_db.forms import OpenIDConnectConfigForm
from mozilla_django_oidc_db.models import OpenIDConnectConfig
from mozilla_django_oidc_db.setup_configuration.models import (
    AdminOIDCConfigurationModel,
    OIDCDiscoveryEndpoint,
)
from mozilla_django_oidc_db.utils import get_groups_by_name


class AdminOIDCConfigurationStep(BaseConfigurationStep[AdminOIDCConfigurationModel]):
    """
    Configure the necessary settings to enable OpenID Connect authentication for admin users.

    This allows admin users to log in with Single Sign On (SSO) to access the management interface.
    """

    verbose_name = "Configuration for admin login via OpenID Connect"
    config_model = AdminOIDCConfigurationModel
    namespace = "oidc_db_config_admin_auth"
    enable_setting = "oidc_db_config_enable"

    def execute(self, model: AdminOIDCConfigurationModel) -> None:
        if len(model.items) != 1:
            raise ConfigurationRunFailed(
                "You must specify exactly one OIDC configuration"
            )

        config_model = model.items[0]

        all_settings = {
            "enabled": config_model.enabled,
            "oidc_rp_client_id": config_model.oidc_rp_client_id,
            "oidc_rp_client_secret": config_model.oidc_rp_client_secret,
            "oidc_rp_sign_algo": config_model.oidc_rp_sign_algo,
            "oidc_rp_scopes_list": config_model.oidc_rp_scopes_list,
            "oidc_token_use_basic_auth": config_model.oidc_token_use_basic_auth,
            "oidc_rp_idp_sign_key": config_model.oidc_rp_idp_sign_key,
            "oidc_use_nonce": config_model.oidc_use_nonce,
            "oidc_nonce_size": config_model.oidc_nonce_size,
            "oidc_state_size": config_model.oidc_state_size,
            "oidc_keycloak_idp_hint": config_model.oidc_keycloak_idp_hint,
            "userinfo_claims_source": config_model.userinfo_claims_source,
            "username_claim": config_model.username_claim,
            "claim_mapping": config_model.claim_mapping,
            "groups_claim": config_model.groups_claim,
            "sync_groups": config_model.sync_groups,
            "sync_groups_glob_pattern": config_model.sync_groups_glob_pattern,
            "make_users_staff": config_model.make_users_staff,
            "superuser_group_names": config_model.superuser_group_names,
            "default_groups": get_groups_by_name(
                config_model.default_groups,
                config_model.sync_groups_glob_pattern,
                config_model.sync_groups,
            ),
        }

        if isinstance(config_model.endpoint_config, OIDCDiscoveryEndpoint):
            all_settings.update(
                oidc_op_discovery_endpoint=config_model.endpoint_config.oidc_op_discovery_endpoint,
            )
        else:
            all_settings.update(
                oidc_op_authorization_endpoint=config_model.endpoint_config.oidc_op_authorization_endpoint,
                oidc_op_token_endpoint=config_model.endpoint_config.oidc_op_token_endpoint,
                oidc_op_user_endpoint=config_model.endpoint_config.oidc_op_user_endpoint,
                oidc_op_logout_endpoint=config_model.endpoint_config.oidc_op_logout_endpoint,
                oidc_op_jwks_endpoint=config_model.endpoint_config.oidc_op_jwks_endpoint,
            )

        form = OpenIDConnectConfigForm(
            instance=OpenIDConnectConfig.get_solo(),
            data=all_settings,
        )
        if not form.is_valid():
            raise ConfigurationRunFailed(
                "Admin OIDC configuration field validation failed",
                form.errors.as_json(),
            )
        form.save()



# Generated by Django 3.2.12 on 2022-03-07 10:28

from django.db import migrations, models

import django_jsonform.models.fields


class Migration(migrations.Migration):
    dependencies = [
        ("mozilla_django_oidc_db", "0006_openidconnectconfig_unique_id_claim"),
    ]

    operations = [
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_exempt_urls",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(max_length=1000, verbose_name="Exempt URL"),
                blank=True,
                default=list,
                help_text="This is a list of absolute url paths, regular expressions for url paths, or Django view names. This plus the mozilla-django-oidc urls are exempted from the session renewal by the SessionRefresh middleware.",
                size=None,
                verbose_name="URLs exempt from session renewal",
            ),
        ),
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_nonce_size",
            field=models.PositiveIntegerField(
                default=32,
                help_text="Sets the length of the random string used for OpenID Connect nonce verification",
                verbose_name="Nonce size",
            ),
        ),
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_state_size",
            field=models.PositiveIntegerField(
                default=32,
                help_text="Sets the length of the random string used for OpenID Connect state verification",
                verbose_name="State size",
            ),
        ),
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_use_nonce",
            field=models.BooleanField(
                default=True,
                help_text="Controls whether the OpenID Connect client uses nonce verification",
                verbose_name="Use nonce",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-07-01 15:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "mozilla_django_oidc_db",
            "0003_openidconnectconfig_oidc_keycloak_idp_hint_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="openidconnectconfig",
            name="oidc_exempt_urls",
        ),
    ]



from typing import Literal, Union

from django_setup_configuration.fields import DjangoModelRef
from django_setup_configuration.models import ConfigurationModel
from pydantic import AnyUrl, Discriminator, Field, Tag
from typing_extensions import Annotated

from mozilla_django_oidc_db.models import OpenIDConnectConfig

EXAMPLE_REALM = "http://keycloak.local:8080/realms/test"


class OIDCFullEndpointConfig(ConfigurationModel):
    oidc_op_authorization_endpoint: AnyUrl = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_authorization_endpoint",
        examples=[f"{EXAMPLE_REALM}/openid-connect/auth"],
    )
    oidc_op_token_endpoint: AnyUrl = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_token_endpoint",
        examples=[f"{EXAMPLE_REALM}/protocol/openid-connect/token"],
    )
    oidc_op_user_endpoint: AnyUrl = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_user_endpoint",
        examples=[f"{EXAMPLE_REALM}/protocol/openid-connect/userinfo"],
    )
    oidc_op_logout_endpoint: AnyUrl | Literal[""] = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_logout_endpoint",
        examples=[f"{EXAMPLE_REALM}/protocol/openid-connect/logout"],
    )
    oidc_op_jwks_endpoint: AnyUrl | Literal[""] = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_jwks_endpoint",
        examples=[f"{EXAMPLE_REALM}/protocol/openid-connect/certs"],
    )


class OIDCDiscoveryEndpoint(ConfigurationModel):
    oidc_op_discovery_endpoint: AnyUrl = DjangoModelRef(
        OpenIDConnectConfig,
        "oidc_op_discovery_endpoint",
        examples=[f"{EXAMPLE_REALM}/"],
    )


def get_endpoint_endpoint_model(endpoint_data):

    if isinstance(endpoint_data, dict):
        discovery_endpoint = endpoint_data.get("oidc_op_discovery_endpoint")
    else:
        discovery_endpoint = getattr(endpoint_data, "oidc_op_discovery_endpoint", None)
    if discovery_endpoint:
        return "discovery"
    return "all"


EndpointConfigUnion = Annotated[
    Union[
        Annotated[OIDCFullEndpointConfig, Tag("all")],
        Annotated[OIDCDiscoveryEndpoint, Tag("discovery")],
    ],
    Discriminator(get_endpoint_endpoint_model),
]


class AdminOIDCConfigurationModelItem(ConfigurationModel):
    # Currently unused because we use a SingletonModel, but this will be relevant in the
    # future
    identifier: str = Field(
        description="a unique identifier for this configuration",
        examples=["admin-oidc"],
    )

    # Change default to True
    enabled: bool = DjangoModelRef(OpenIDConnectConfig, "enabled", default=True)

    # Json
    claim_mapping: dict = DjangoModelRef(OpenIDConnectConfig, "claim_mapping")

    # Arrays are overridden to make the typing simpler (the underlying Django field is an ArrayField, which is non-standard)
    username_claim: list[str] = DjangoModelRef(
        OpenIDConnectConfig,
        "username_claim",
        examples=[["nested", "username", "claim"]],
    )
    groups_claim: list[str] = DjangoModelRef(OpenIDConnectConfig, "groups_claim")
    superuser_group_names: list[str] = DjangoModelRef(
        OpenIDConnectConfig, "superuser_group_names", examples=[["superusers"]]
    )
    default_groups: list[str] = DjangoModelRef(
        OpenIDConnectConfig,
        "default_groups",
        examples=[["read-only-users"]],
        default=list,
    )
    oidc_rp_scopes_list: list[str] = DjangoModelRef(
        OpenIDConnectConfig, "oidc_rp_scopes_list"
    )

    endpoint_config: EndpointConfigUnion

    class Meta:
        django_model_refs = {
            OpenIDConnectConfig: [
                "oidc_rp_client_id",
                "oidc_rp_client_secret",
                "oidc_token_use_basic_auth",
                "oidc_rp_sign_algo",
                "oidc_rp_idp_sign_key",
                "oidc_use_nonce",
                "oidc_nonce_size",
                "oidc_state_size",
                "oidc_keycloak_idp_hint",
                "userinfo_claims_source",
                "sync_groups",
                "sync_groups_glob_pattern",
                "make_users_staff",
            ]
        }
        extra_kwargs = {
            "oidc_rp_client_id": {"examples": ["modify-this"]},
            "oidc_rp_client_secret": {"examples": ["modify-this"]},
            "oidc_rp_idp_sign_key": {"examples": ["modify-this"]},
            "oidc_keycloak_idp_hint": {"examples": ["some-identity-provider"]},
        }


class AdminOIDCConfigurationModel(ConfigurationModel):
    items: list[AdminOIDCConfigurationModelItem]
